@@ -18,7 +18,7 @@ extern int hwload_mode;
 int sighandler_pipe[2] = { 0, 0 };
 
 // Timespec utility function
-void timespec_add(struct timespec* timespec, long nanoseconds){
+void timespec_add(struct timespec* timespec, int64_t nanoseconds){
     nanoseconds += timespec->tv_nsec;
     timespec->tv_sec += nanoseconds / 1000000000;
     timespec->tv_nsec = nanoseconds % 1000000000;
@@ -290,6 +290,11 @@ int main(int argc, char** argv){
 
     srand(time(NULL));
 
+    if(init_cond_monotonic())  {
+        ckb_fatal("Failed to initialize monotonic clock.");
+        exit(1);
+    }
+
     // Start the USB system
     int result = usbmain();
     quit();
