@@ -40,7 +40,19 @@ typedef struct {
     macroaction* actions;
     int actioncount;
     uchar combo[N_KEYBYTES_INPUT];
+
+    /*
+     * The number of times the key has been pressed and released.  It is therefore
+     * even if the key is up, and odd if the key is down.  Whenever the macro execution
+     * completes:
+     *  - The counter is decremented by two, unless it is at 1.
+     *  - If state > 0, the macro is repeated, with a delay if state == 1.
+     *
+     * If the key is released during the delay, the counter is set to 0, and the running
+     * thread shuts itself down immediately.
+     */
     char triggered;
+    void* running;      // pointer to the parameter_t of the currently running thread (null if none)
 } keymacro;
 
 // Key bindings for a mode (keyboard + mouse)
@@ -205,6 +217,8 @@ typedef struct {
     int uinput_kb, uinput_mouse;
     // keyboard led thread
     pthread_t* ledthread;
+    // Hack to get the led thread to shut down if stuck waiting for dmutex
+    int shutdown_ledthread;
 #else
     // USB identifier
     uint32_t location_id[IFACE_MAX + 1];
