@@ -18,7 +18,7 @@ static char kbsyspath[DEV_MAX][FILENAME_MAX];
 int os_usbsend_control(usbdevice* kb, uchar* data, ushort len, uchar bRequest, ushort wValue, ushort wIndex, const char* file, int line) {
 #ifdef DEBUG_USB_SEND
     int ckb = INDEX_OF(kb, keyboard);
-    ckb_info("ckb%d Control (%s:%d): bmRequestType: 0x%02hhx, bRequest: %hhu, wValue: 0x%04hx, wIndex: %04hx, wLength: %hu\n", ckb, file, line, 0x40, bRequest, wValue, wIndex, len);
+    ckb_info("ckb%d Control (%s:%d): bmRequestType: 0x%02hhx, bRequest: %hhu, wValue: 0x%04hx, wIndex: %04hx, wLength: %hu", ckb, file, line, 0x40, bRequest, wValue, wIndex, len);
     if(len)
         print_urb_buffer("Control buffer:", data, len, file, line, __func__, ckb);
 #endif
@@ -27,14 +27,14 @@ int os_usbsend_control(usbdevice* kb, uchar* data, ushort len, uchar bRequest, u
     int res = ioctl(kb->handle - 1, USBDEVFS_CONTROL, &transfer);
     if (res == -1){
         int ioctlerrno = errno;
-        ckb_err_fn(" %s, res = 0x%x\n", file, line, strerror(ioctlerrno), res);
+        ckb_err_fn(" %s, res = 0x%x", file, line, strerror(ioctlerrno), res);
         if(ioctlerrno == ETIMEDOUT)
             return -1;
         else
             return 0;
 
     } else if (res != len)
-        ckb_warn_fn("Wrote %d bytes (expected %d)\n", file, line, res, MSG_SIZE);
+        ckb_warn_fn("Wrote %d bytes (expected %d)", file, line, res, MSG_SIZE);
 
     return res;
 }
@@ -98,7 +98,7 @@ int os_usbsend(usbdevice* kb, const uchar* out_msg, int is_recv, const char* fil
         // If we need to read a response, lock the interrupt mutex
         if(is_recv)
             if(pthread_mutex_lock(intmutex(kb)))
-                ckb_fatal("Error locking interrupt mutex in os_usbsend()\n");
+                ckb_fatal("Error locking interrupt mutex in os_usbsend()");
 
         struct usbdevfs_bulktransfer transfer = {0};
         // All firmware versions for normal HID devices have the OUT endpoint at the end
@@ -116,7 +116,7 @@ int os_usbsend(usbdevice* kb, const uchar* out_msg, int is_recv, const char* fil
 
     if (res <= 0){
         int ioctlerrno = errno;
-        ckb_err_fn(" %s, res = 0x%x\n", file, line, res ? strerror(ioctlerrno) : "No data written", res);
+        ckb_err_fn("%s, res = 0x%x", file, line, res ? strerror(ioctlerrno) : "No data written", res);
         if(res == -1 && ioctlerrno == ETIMEDOUT){
             if(is_recv)
                 pthread_mutex_unlock(intmutex(kb));
@@ -127,7 +127,7 @@ int os_usbsend(usbdevice* kb, const uchar* out_msg, int is_recv, const char* fil
             return 0;
         }
     } else if (res != MSG_SIZE)
-        ckb_warn_fn("Wrote %d bytes (expected %d)\n", file, line, res, MSG_SIZE);
+        ckb_warn_fn("Wrote %d bytes (expected %d)", file, line, res, MSG_SIZE);
 #ifdef DEBUG_USB_SEND
     print_urb_buffer("Sent:", out_msg, MSG_SIZE, file, line, __func__, INDEX_OF(kb, keyboard));
 #endif
@@ -169,36 +169,34 @@ int os_usbrecv(usbdevice* kb, uchar* in_msg, const char* file, int line){
     int res;
     if(kb->fwversion >= 0x120 || IS_V2_OVERRIDE(kb)){
         // Wait for 2s
-        struct timespec condwait = {0};
-        condwait.tv_sec = time(NULL) + 2;
-        int condret = pthread_cond_timedwait(intcond(kb), intmutex(kb), &condwait);
+        int condret = cond_nanosleep(intcond(kb), intmutex(kb), 2000000000);
         if(condret != 0){
             if(pthread_mutex_unlock(intmutex(kb)))
-                ckb_fatal("Error unlocking interrupt mutex in os_usbrecv()\n");
+                ckb_fatal("Error unlocking interrupt mutex in os_usbrecv()");
             if(condret == ETIMEDOUT)
-                ckb_warn_fn("ckb%d: Timeout while waiting for response\n", file, line, INDEX_OF(kb, keyboard));
+                ckb_warn_fn("ckb%d: Timeout while waiting for response", file, line, INDEX_OF(kb, keyboard));
             else
-                ckb_warn_fn("Interrupt cond error %i\n", file, line, condret);
+                ckb_warn_fn("Interrupt cond error %i", file, line, condret);
             return -1;
         }
         memcpy(in_msg, kb->interruptbuf, MSG_SIZE);
         memset(kb->interruptbuf, 0, MSG_SIZE);
         res = MSG_SIZE;
         if(pthread_mutex_unlock(intmutex(kb)))
-            ckb_fatal("Error unlocking interrupt mutex in os_usbrecv()\n");
+            ckb_fatal("Error unlocking interrupt mutex in os_usbrecv()");
     } else {
         struct usbdevfs_ctrltransfer transfer = { 0xa1, 0x01, 0x0300, kb->epcount - 1, MSG_SIZE, 5000, in_msg };
         res = ioctl(kb->handle - 1, USBDEVFS_CONTROL, &transfer);
         if(res <= 0){
             // This is done because ckb_err_fn can set errno itself
             int ioctlerrno = errno;
-            ckb_err_fn("%s\n", file, line, res ? strerror(ioctlerrno) : "No data read");
+            ckb_err_fn("%s", file, line, res ? strerror(ioctlerrno) : "No data read");
             if(res == -1 && ioctlerrno == ETIMEDOUT)
                 return -1;
             else
                 return 0;
         } else if(res != MSG_SIZE)
-            ckb_warn_fn("Read %d bytes (expected %d)\n", file, line, res, MSG_SIZE);
+            ckb_warn_fn("Read %d bytes (expected %d)", file, line, res, MSG_SIZE);
     }
 
 #ifdef DEBUG_USB_RECV
@@ -242,7 +240,7 @@ int _nk95cmd(usbdevice* kb, uchar bRequest, ushort wValue, const char* file, int
     struct usbdevfs_ctrltransfer transfer = { 0x40, bRequest, wValue, 0, 0, 5000, 0 };
     int res = ioctl(kb->handle - 1, USBDEVFS_CONTROL, &transfer);
     if(res <= 0){
-        ckb_err_fn("%s\n", file, line, res ? strerror(errno) : "No data written");
+        ckb_err_fn("%s", file, line, res ? strerror(errno) : "No data written");
         return 1;
     }
     return 0;
@@ -273,9 +271,11 @@ void os_sendindicators(usbdevice* kb) {
         ileds = &kb->ileds;
     }
     struct usbdevfs_ctrltransfer transfer = { 0x21, 0x09, 0x0200, 0x00, ((kb->fwversion >= 0x300 || IS_V3_OVERRIDE(kb)) ? 2 : 1), 500, ileds };
+    queued_mutex_unlock(dmutex(kb));
     int res = ioctl(kb->handle - 1, USBDEVFS_CONTROL, &transfer);
+    queued_mutex_lock(dmutex(kb));
     if(res <= 0) {
-        ckb_err("%s\n", res ? strerror(errno) : "No data written");
+        ckb_err("%s", res ? strerror(errno) : "No data written");
         if (usb_tryreset(kb) == 0 && countForReset++ < 3) {
             os_sendindicators(kb);
         }
@@ -300,7 +300,7 @@ void* os_inputmain(void* context){
     int fd = kb->handle - 1;
     ushort vendor = kb->vendor, product = kb->product;
     int index = INDEX_OF(kb, keyboard);
-    ckb_info("Starting input thread for %s%d\n", devpath, index);
+    ckb_info("Starting input thread for %s%d", devpath, index);
 
     /// Here the actions in detail:
     ///
@@ -308,7 +308,7 @@ void* os_inputmain(void* context){
     /// For RGB, monitor all but the last, as it's used for input/output
     int urbcount = ((IS_LEGACY(vendor, product) || product == P_ST100) ? kb->epcount : (kb->epcount - 1));
     if (urbcount == 0) {
-        ckb_err("urbcount = 0, so there is nothing to claim in os_inputmain()\n");
+        ckb_err("urbcount = 0, so there is nothing to claim in os_inputmain()");
         return 0;
     }
 
@@ -344,7 +344,7 @@ void* os_inputmain(void* context){
         // usbX/X-X/X-X:1.0/ep_80
         // ~~~~~~~~~~~~~~~~~~~^
         if(path[strlen(path) - 3] == '_'){
-            ckb_info("Applying udev endpoint workaround for %s\n", path);
+            ckb_info("Applying udev endpoint workaround for %s", path);
             // Skip the current entry
             udeventry = nextentry;
             nextentry = udev_list_entry_get_next(udeventry);
@@ -366,9 +366,9 @@ void* os_inputmain(void* context){
         if(sizehex)
             sscanf(sizehex, "%hx", &size);
         else
-            ckb_warn("Unable to read wMaxPacketSize for %s, assuming 64\n", epstr);
+            ckb_warn("Unable to read wMaxPacketSize for %s, assuming 64", epstr);
 #ifdef DEBUG
-        ckb_info("Endpoint path %s has wMaxPacketSize %i\n", epstr, size);
+        ckb_info("Endpoint path %s has wMaxPacketSize %i", epstr, size);
 #endif
         // Increment the udev list pointer
         udeventry = nextentry;
@@ -423,7 +423,7 @@ void* os_inputmain(void* context){
     ///
     /// If the endless loop is terminated, clean up by discarding the URBs via ioctl(USBDEVFS_DISCARDURB),
     /// free the URB buffers and return a null pointer as thread exit code.
-    ckb_info("Stopping input thread for %s%d\n", devpath, index);
+    ckb_info("Stopping input thread for %s%d", devpath, index);
     for(int i = 0; i < urbcount; i++){
         ioctl(fd, USBDEVFS_DISCARDURB, urbs + i);
         free(urbs[i].buffer);
@@ -456,8 +456,8 @@ static int usbunclaim(usbdevice* kb, int resetting) {
     }
     // Intentional unclean exit workaround, because usbhid hangs while initialising these devices.
     if (NEEDS_UNCLEAN_EXIT(kb)) {
-        ckb_warn("Your %s is being uncleanly removed to speed up shutdown times.\n", kb->name);
-        ckb_warn("If you still need the device, you will have to restart ckb-next-daemon.\n");
+        ckb_warn("Your %s is being uncleanly removed to speed up shutdown times.", kb->name);
+        ckb_warn("If you still need the device, you will have to restart ckb-next-daemon.");
         return 0;
     }
     // For RGB keyboards, the kernel driver should only be reconnected to interfaces 0 and 1 (HID), and only if we're not about to do a USB reset.
@@ -510,14 +510,14 @@ void os_closeusb(usbdevice* kb){
 static int usbclaim(usbdevice* kb){
     int count = kb->epcount;
 #ifdef DEBUG
-    ckb_info("claiming %d endpoints\n", count);
+    ckb_info("claiming %d endpoints", count);
 #endif // DEBUG
 
     for(int i = 0; i < count; i++){
         struct usbdevfs_ioctl ctl = { i, USBDEVFS_DISCONNECT, 0 };
         ioctl(kb->handle - 1, USBDEVFS_IOCTL, &ctl);
         if(ioctl(kb->handle - 1, USBDEVFS_CLAIMINTERFACE, &i)) {
-            ckb_err("Failed to claim interface %d: %s\n", i, strerror(errno));
+            ckb_err("Failed to claim interface %d: %s", i, strerror(errno));
             return -1;
         }
     }
@@ -529,7 +529,7 @@ static int usbclaim(usbdevice* kb){
 ///
 #define TEST_RESET(op)                                                      \
     if(op){                                                                 \
-        ckb_err_fn("resetusb failed: %s\n", file, line, strerror(errno));   \
+        ckb_err_fn("resetusb failed: %s", file, line, strerror(errno));   \
         if(errno == EINTR || errno == EAGAIN)                               \
             return -1;              /* try again if status code says so */  \
         return -2;                  /* else, remove device */               \
@@ -559,19 +559,27 @@ int os_resetusb(usbdevice* kb, const char* file, int line) {
 /// \param string
 ///
 void strtrim(char* string){
-    // Find last non-space
-    char* last = string;
-    for(char* c = string; *c != 0; c++){
-        if(!isspace(*c))
-            last = c;
-    }
-    last[1] = 0;
     // Find first non-space
     char* first = string;
     for(; *first != 0; first++){
         if(!isspace(*first))
             break;
     }
+
+    // If we reached the end and didn't find anything, blank out the string
+    if(first[1] == '\0'){
+        string[0] = '\0';
+        return;
+    }
+
+    // Find last non-space
+    char* last = first;
+    for(char* c = string; *c != 0; c++){
+        if(!isspace(*c))
+            last = c;
+    }
+    last[1] = '\0';
+
     if(first != string)
         memmove(string, first, last - first);
 }
@@ -605,7 +613,7 @@ int os_setupusb(usbdevice* kb) {
     int index = INDEX_OF(kb, keyboard);
 
     /// - Do some output about connecting interfaces
-    ckb_info("Connecting %s at %s%d\n", kb->name, devpath, index);
+    ckb_info("Connecting %s at %s%d", kb->name, devpath, index);
 
     ///
     /// - Claim the USB interfaces
@@ -614,7 +622,7 @@ int os_setupusb(usbdevice* kb) {
     ///
     const char* ep_str = udev_device_get_sysattr_value(dev, "bNumInterfaces");
 #ifdef DEBUG
-    ckb_info("Claiming interfaces. name=%s, firmware=%s, ep_str=%s\n", name, firmware, ep_str);
+    ckb_info("Claiming interfaces. name=%s, firmware=%s, ep_str=%s", name, firmware, ep_str);
 #endif //DEBUG
     kb->epcount = 0;
     if(ep_str)
@@ -622,11 +630,11 @@ int os_setupusb(usbdevice* kb) {
     if(kb->epcount < 2 && !IS_SINGLE_EP(kb)){
         // If we have an RGB KB with 1 endpoint, it will be in BIOS mode.
         if(kb->epcount == 1){
-            ckb_info("Device is in BIOS mode\n");
+            ckb_info("Device is in BIOS mode");
             return -1;
         }
         // Something probably went wrong if we got here
-        ckb_err("Unable to read endpoint count from udev, assuming %d\n", kb->epcount);
+        ckb_err("Unable to read endpoint count from udev, assuming %d", kb->epcount);
         if (usb_tryreset(kb) == 0) { ///< Try to reset the device and recall the function
             static int retryCount = 0; ///< Don't do this endless in recursion
             if (retryCount++ < 5) {
@@ -635,45 +643,66 @@ int os_setupusb(usbdevice* kb) {
         }
         return -1;
         // ToDo are there special versions we have to detect? If there are, that was the old code to handle it:
-        // This shouldn't happen, but if it does, assume EP count based onckb_warn what the device is supposed to have
+        // This shouldn't happen, but if it does, assume EP count based on what the device is supposed to have
         // kb->epcount = (HAS_FEATURES(kb, FEAT_RGB) ? 4 : 3);
-        // ckb_warn("Unable to read endpoint count from udev, assuming %d and reading >>%s<<...\n", kb->epcount, ep_str);
+        // ckb_warn("Unable to read endpoint count from udev, assuming %d and reading >>%s<<...", kb->epcount, ep_str);
     }
     if(usbclaim(kb)){
-        ckb_err("Failed to claim interfaces: %s\n", strerror(errno));
+        ckb_err("Failed to claim interfaces: %s", strerror(errno));
         return -1;
     }
     return 0;
 }
 
 int usbadd(struct udev_device* dev, ushort vendor, ushort product) {
+    // Check if device should be ignored
+    for(int i = 0; i < DEV_MAX; i++){
+        if(ignored_devices[i].idVendor == vendor && ignored_devices[i].idProduct == product){
+            ckb_info("Ignoring device 0x%hx:0x%hx as requested. If you're using this to work around a bug, please report it.", vendor, product);
+            return 1;
+        }
+    }
     const char* path = udev_device_get_devnode(dev);
     const char* syspath = udev_device_get_syspath(dev);
     if(!path || !syspath || path[0] == 0 || syspath[0] == 0){
-        ckb_err("Failed to get device path\n");
+        ckb_err("Failed to get device path");
         return -1;
     }
 #ifdef DEBUG
-    ckb_info(">>>vendor = 0x%x, product = 0x%x, path = %s, syspath = %s\n", vendor, product, path, syspath);
+    ckb_info(">>>vendor = 0x%x, product = 0x%x, path = %s, syspath = %s", vendor, product, path, syspath);
 #endif // DEDBUG
+
+    for(int i = 0; i < DEV_MAX; i++){
+        // Make sure any existing keyboard doesn't have the same syspath (this shouldn't happen, but it does happen...)
+        // We have to do it separately in case a device was unplugged and there's an empty usbdevice
+        if(!strcmp(syspath, kbsyspath[i])){
+            ckb_warn("Attempted to add device with duplicate syspath %s (path %s)", syspath, path);
+            return 1;
+        }
+    }
+
     // Find a free USB slot
     for(int index = 1; index < DEV_MAX; index++){
         usbdevice* kb = keyboard + index;
-        if(pthread_mutex_trylock(dmutex(kb))){
+        if(queued_mutex_trylock(dmutex(kb))){
             // If the mutex is locked then the device is obviously in use, so keep going
-            if(!strcmp(syspath, kbsyspath[index])){
-                // Make sure this existing keyboard doesn't have the same syspath (this shouldn't happen)
-                return 0;
-            }
             continue;
         }
-        if(!IS_CONNECTED(kb)){
+        // We can't use IS_CONNECTED() here.
+        // If multiple devices are being opened at once, we will attempt to set up the second one before
+        // we're done with the first one, which means the uinput handles might still be 0.
+        // This will cause IS_CONNECTED to return 0, resulting in overwriting ckb1's kb fields with ckb2's,
+        // leaving ckb1 in an unknown and possibly uninitialised state.
+        //
+        // This happened rarely because dmutex was most likely already locked, so we just skipped over the device
+        // before getting here. (See trylock above.)
+        if(!kb->handle){
             // Open the sysfs device
             kb->handle = open(path, O_RDWR) + 1;
             if(kb->handle <= 0){
-                ckb_err("Failed to open USB device: %s\n", strerror(errno));
+                ckb_err("Failed to open USB device: %s", strerror(errno));
                 kb->handle = 0;
-                pthread_mutex_unlock(dmutex(kb));
+                queued_mutex_unlock(dmutex(kb));
                 return -1;
             } else {
                 // Set up device
@@ -686,9 +715,9 @@ int usbadd(struct udev_device* dev, ushort vendor, ushort product) {
                 return 0;
             }
         }
-        pthread_mutex_unlock(dmutex(kb));
+        queued_mutex_unlock(dmutex(kb));
     }
-    ckb_err("No free devices\n");
+    ckb_err("No free devices");
     return -1;
 }
 
@@ -746,10 +775,10 @@ static void usb_rm_device(struct udev_device* dev){
     if(!syspath || syspath[0] == 0)
         return;
     for(int i = 1; i < DEV_MAX; i++){
-        pthread_mutex_lock(devmutex + i);
+        queued_mutex_lock(devmutex + i);
         if(!strcmp(syspath, kbsyspath[i]))
             closeusb(keyboard + i);
-        pthread_mutex_unlock(devmutex + i);
+        queued_mutex_unlock(devmutex + i);
     }
 }
 
@@ -806,7 +835,7 @@ static time_t get_clock_monotonic_seconds() {
     struct timespec timespec_var = {0};
 
     if(clock_gettime(CLOCK_BOOTTIME, &timespec_var))
-        ckb_err("Error in clock_gettime()\n");
+        ckb_err("Error in clock_gettime()");
 
     return timespec_var.tv_sec;
 }
@@ -837,7 +866,7 @@ int usbmain(){
     /// Create the udev object with udev_new() (is a function from libudev.h)
     /// terminate -1 if error
     if(!(udev = udev_new())) {
-        ckb_fatal("Failed to initialize udev in usbmain(), usb_linux.c\n");
+        ckb_fatal("Failed to initialize udev in usbmain(), usb_linux.c");
         return -1;
     }
     
@@ -879,8 +908,10 @@ int usbmain(){
                 if(res == 0)
                     continue;
                 // If the device matched but the handle wasn't opened correctly, re-enumerate (this sometimes solves the problem)
-                if(res == -1)
+                if(res == -1){
+                    ckb_warn("Handle wasn't opened correctly. Trying again");
                     udev_enum();
+                }
             } else if(!strcmp(action, "remove"))
                 usb_rm_device(dev);
             udev_device_unref(dev);
