@@ -22,7 +22,8 @@ Kb::Kb(QObject *parent, const QString& path) :
     _currentProfile(0), _currentMode(0), _model(KeyMap::NO_MODEL),
     lastAutoSave(QDateTime::currentMSecsSinceEpoch()),
     _hwProfile(0), prevProfile(0), prevMode(0),
-    cmd(cmdpath), notifyNumber(1), macroNumber(2), _needsSave(false), _layout(KeyMap::NO_LAYOUT), _maxDpi(0)
+    cmd(cmdpath), notifyNumber(1), macroNumber(2), _needsSave(false), _layout(KeyMap::NO_LAYOUT), _maxDpi(0),
+    deviceIdleTimer()
 {
     memset(iState, 0, sizeof(iState));
     memset(hwLoading, 0, sizeof(hwLoading));
@@ -522,6 +523,7 @@ void Kb::readNotify(QString line){
             mode->light()->animKeypress(keyName, keyPressed);
             mode->bind()->keyEvent(keyName, keyPressed);
         }
+        deviceIdleTimer.start();
     } else if(components[0] == "i"){
         // Indicator event
         QString i = components[1];
@@ -808,6 +810,14 @@ void Kb::setCurrentMode(KbMode* mode){
     mode->light()->forceFrameUpdate();
 }
 
+KbProfile* Kb::newProfileWithBlankMode(){
+    KbProfile* p = new KbProfile(this, getKeyMap());
+    KbMode* m = newMode();
+    p->append(m);
+    p->currentMode(m);
+    return p;
+}
+
 KeyMap::Layout Kb::getCurrentLayout(){
     return _layout;
 }
