@@ -5,9 +5,10 @@
 #include <QFile>
 #include <QThread>
 #include "kbprofile.h"
+#include <QElapsedTimer>
+#include <limits>
 
 // Class for managing devices
-
 class Kb : public QThread
 {
     Q_OBJECT
@@ -76,7 +77,7 @@ public:
     void        setCurrentMode(KbMode* mode);
 
     // Create a new profile/mode. The newly-created object will NOT be inserted into the current profile/mode list.
-    inline KbProfile*   newProfile()                                  { return new KbProfile(this, getKeyMap()); }
+    KbProfile*   newProfileWithBlankMode();
     inline KbProfile*   newProfile(KbProfile* other)                  { return new KbProfile(this, getKeyMap(), *other); }
     inline KbProfile*   newProfile(CkbExternalSettings* settings, QString guid) { return new KbProfile(this, getKeyMap(), *settings, guid); }
     inline KbMode*      newMode()                                     { return new KbMode(this, getKeyMap()); }
@@ -113,6 +114,10 @@ public:
     inline ushort getMaxDpi () {return _maxDpi; }
     void setPollRate(QString poll);
 
+    // The valid check is done because we don't start the timer on purpose in the constructor.
+    // This is done so that when a new device is plugged in while the lights are off, it doesn't suddenly return a really low value and wake everything up.
+    inline qint64 getDeviceIdleTime() const { return (deviceIdleTimer.isValid() ? deviceIdleTimer.elapsed() : std::numeric_limits<qint64>::max()); }
+
     ~Kb();
 
 signals:
@@ -220,6 +225,8 @@ private:
     // Notification reader, launches as a separate thread and reads from file.
     // (QFile doesn't have readyRead() so there's no other way to do this asynchronously)
     void run();
+
+    QElapsedTimer deviceIdleTimer;
 };
 
 #endif // KB_H
