@@ -37,7 +37,7 @@ public:
     // Overall dimming. 0 = max brightness, 3 = off
     static const int    MAX_DIM = 3;
     inline int          dimming()               { return _dimming; }
-    void                dimming(int newDimming, bool overrideShare = false, bool noSave = false);
+    void                dimming(int newDimming, bool noSave = false);
     // Shared brightness between all modes (-1 for sharing disabled)
     static int          shareDimming();
     static void         shareDimming(int newShareDimming);
@@ -84,8 +84,9 @@ public:
     // Needed for when switching profiles as the daemon wipes the old data on switch
     void forceFrameUpdate();
 
-    void timerDimRestore();
+    int timerDimRestore();
     void timerDim();
+    inline bool isTimerDimmed() const { return _timerDimmed; }
 signals:
     void didLoad();
     void updated();
