@@ -41,12 +41,15 @@ public:
 
     // Timer for scanning the driver/device list. May also be useful for periodic GUI events. Created during init(), always runs at 10FPS.
     static inline QTimer* scanTimer()       { return _kbManager ? _kbManager->_scanTimer : 0; }
+    inline bool getDeviceTimerDimmed() { for(Kb* kb : _devices) if(kb->currentLight()->isTimerDimmed()) { return true; } return false; }
 
 #ifdef USE_XCB_SCREENSAVER
     // Called to restart the idle timer
     static void setIdleTimer(bool enable);
 #endif
 
+public slots:
+    void brightnessScroll(int delta, Qt::Orientation orientation);
 signals:
     // A new device was connected.
     void kbConnected(Kb* device);
@@ -73,6 +76,7 @@ private:
 #ifdef USE_XCB_SCREENSAVER
     static QTimer* _idleTimer;
 #endif
+    int getLastUsedDeviceIdleTime();
 };
 
 #endif // KBMANAGER_H
