@@ -15,12 +15,23 @@
 
 #endif
 
+// Mute device selection is supported
+EXTERN_C bool isMuteDeviceSupported();
+
+// Device class for mute state
+ENUM_C(muteDevice) {
+    SINK,
+    SOURCE
+} ENUM_END_C(muteDevice);
+
 // Gets the default audio device's mute state
 ENUM_C(muteState) {
     UNKNOWN = -1,
     UNMUTED,
     MUTED
 } ENUM_END_C(muteState);
-EXTERN_C muteState getMuteState();
+
+EXTERN_C muteState getMuteState(const muteDevice muteDev);
+EXTERN_C void deinitAudioSubsystem();
 
 #endif
