@@ -10,6 +10,7 @@
 #include <QDateTime>
 #include "mainwindow.h"
 #include <ckbnextconfig.h>
+#include <QProgressBar>
 
 extern QString devpath;
 
@@ -18,7 +19,7 @@ static QStringList modKeys, modNames;
 static bool updateRequestedByUser = false;
 
 SettingsWidget::SettingsWidget(QWidget *parent) :
-    QWidget(parent), devDetect(nullptr),
+    QWidget(parent), devDetect(nullptr), devDetectProgress(nullptr),
     ui(new Ui::SettingsWidget)
 {
     ui->setupUi(this);
@@ -88,14 +89,16 @@ SettingsWidget::SettingsWidget(QWidget *parent) :
         ui->loginItemBox->setChecked(AutoRun::isEnabled());
     }
 
+    QString copyrightText = QString("© 2014-2016 <a href=\"https://github.com/ccMSC/\" style=\"text-decoration:none;\">ccMSC</a>.<br/>© 2017-%1 <a href=\"https://github.com/ckb-next/ckb-next/graphs/contributors\" style=\"text-decoration:none;\">%2</a>.").arg(CKB_NEXT_COPYRIGHT_YEAR, tr("The ckb-next development team"));
+
 #ifndef OS_MAC_LEGACY
 #ifdef Q_OS_MACOS
-    QString labelText = ui->label_2->text();
-    labelText.append(tr("<br/>Special thanks to <a href=\"https://github.com/tekezo\" style=\"text-decoration:none;\">tekezo</a> for <a href=\"https://github.com/tekezo/Karabiner-VirtualHIDDevice\" style=\"text-decoration:none;\">VirtualHIDDevice</a>."));
-    ui->label_2->setText(labelText);
+    copyrightText.append(tr("<br/>Special thanks to <a href=\"https://github.com/tekezo\" style=\"text-decoration:none;\">tekezo</a> for <a href=\"https://github.com/tekezo/Karabiner-VirtualHIDDevice\" style=\"text-decoration:none;\">VirtualHIDDevice</a>."));
 #endif
 #endif
 
+    ui->label_2->setText(copyrightText);
+
 #ifdef DISABLE_UPDATER
     ui->autoUpdBox->hide();
     ui->pushButton_2->hide();
@@ -188,7 +191,7 @@ void SettingsWidget::on_aboutQt_clicked(){
 
 void SettingsWidget::on_generateReportButton_clicked(){
     // Don't allow the script to run twice
-    if(devDetect)
+    if(devDetect || devDetectProgress)
         return;
 
     QMessageBox::information(this, tr("Generate report"), tr("This will collect software logs, as well as information about the Corsair devices in your system.\n\n"
@@ -208,10 +211,17 @@ void SettingsWidget::on_generateReportButton_clicked(){
         QMessageBox::critical(this, tr("Error executing ckb-next-dev-detect"), tr("An error occurred while trying to execute ckb-next-dev-detect.\n"
                                                                                   "File not found or not executable."));
         devDetect->deleteLater();
+        return;
     }
+    devDetectProgress = new QProgressDialog(tr("Generating Report"), "", 0, 0, this);
+    devDetectProgress->setCancelButton(nullptr);
+    devDetectProgress->open();
 }
 
 void SettingsWidget::devDetectFinished(int retVal){
+    devDetectProgress->close();
+    delete devDetectProgress;
+    devDetectProgress = nullptr;
     QFile report("/tmp/ckb-next-dev-detect-report.gz");
     if(retVal || !report.exists()){
         QString errMsg(tr("An error occurred while trying to execute ckb-next-dev-detect.\n\n"));
